from . import log_support as logsup
from abc import abstractmethod
from . import global_support as gsup
from fabric import Connection

class SshMan(object):
	
	def __init__(self,fabric_connection: Connection,remote_workdir: gsup.Folder):
		self.conn: Connection=fabric_connection
		# if isinstance(remote_workdir,str): remote_workdir=remote_projects_dir.create(
		# 	remote_workdir)
		assert isinstance(remote_workdir,
		                  gsup.Folder),f"'remote_workdir' is neither a Folder nor a string."
		self.workdir: gsup.Folder=remote_workdir
		if self.conn.run(f'[ -d "{remote_workdir.path}" ] && echo "exist" || echo "not exist"',warn=True).stdout.strip()=='not exist':
			logsup.logger.info('Creating folder %s on remote host.',remote_workdir.path)
			self.conn.run(f'mkdir -p {remote_workdir.path}')
	
	def check_exists(self,program_name):
		which_result=self.conn.run(f'which {program_name}',warn=True)
		if which_result.failed: return False
		else:
			logsup.logger.info(f'"{program_name}" already exists.')
			return True
	
	def apt_install(self,*package_names,forced=False):
		names=list()
		for package_name in package_names:
			pack_names=[x for x in package_name.split(" ") if x]
			names.extend(pack_names)
		
		if not forced:
			for package_name in names.copy():
				if self.check_exists(package_name):
					names.remove(package_name)
					logsup.logger.info('Skipping installation of "%s".',package_name)
		
		for package_name in names:
			self.run(f'sudo apt-get install {package_name} -y')
			package_path=self.conn.run(f'which {package_name}',warn=True).stdout.strip()
			logsup.logger.debug('"%s" path: %s',package_name,package_path)
	
	@abstractmethod
	def initial_setup(self,*args,**kwargs):
		pass
	
	@abstractmethod
	def execute(self,*args,**kwargs):
		pass
	
	def run(self,command,**kwargs):
		logsup.logger.info('Running !!!%s!!!',command)
		return self.conn.run(command,**kwargs)
	
	def put(self,local,remote,preserve_mode=True):
		logsup.logger.info('Putting from !!!%s!!! to !!!%s!!!',local,remote)
		self.conn.put(local,remote=remote,preserve_mode=preserve_mode)
	
	def get(self,remote,local=None,preserve_mode=True):
		logsup.logger.info('Getting from !!!%s!!! to !!!%s!!!',remote,local)
		self.conn.get(remote,local=local,preserve_mode=preserve_mode)
	
	def cd(self,path):
		return self.conn.cd(path=path)
	
	def cd_to_working_dir(self):
		return self.cd(self.workdir.path)
	
	def prefix(self,command):
		return self.conn.prefix(command=command)
	
	def sudo(self,command,**kwargs):
		return self.conn.sudo(command=command,**kwargs)

def main():
	pass

if __name__=='__main__':
	main()



import os
import numpy as np
import pandas as pd
import itertools
import collections
from telegram.ext import Updater,InlineQueryHandler,CommandHandler
import requests
import re
from telegram import Bot

notigistic_bot=Bot(r'883796622:AAE4eSIVACAIph2l_8124OhilKdRMQMZ1gY')

class TheChat(object):
	
	def __init__(self,chat_id,bot=None):
		self.chat_id=chat_id
		self.bot=bot or notigistic_bot
	
	def send_message(self,message):
		self.bot.send_message(chat_id=self.chat_id,text=message)

egistic_notify=TheChat(r'-366778823')

# def get_url():
# 	contents=requests.get('https://random.dog/woof.json').json()
# 	url=contents['url']
# 	return url
#
# def get_image_url():
# 	file_extension=''
# 	url=None
# 	while file_extension not in allowed_extension:
# 		url=get_url()
# 		file_extension=re.search("([^.]*)$",url).group(1).lower()
# 	return url

# def bop(the_bot: Bot,text):
# 	# url=get_image_url()
# 	# the_bot.send_photo(chat_id=chat_id,photo=url)
# 	pass
#
# def egistic_notify_send_text(text):
# 	notigistic_bot.send_message(chat_id=chat_id,text=text)

def main():
	pass

# updater=Updater(token)
# dp=updater.dispatcher
# dp.add_handler(CommandHandler('bop',bop))
# updater.start_polling()
# updater.idle()

if __name__=='__main__':
	main()



import circus
import signal

class TheWatcher(object):
	watchers=list()
	
	def __init__(self,name,cmd,numprocesses=1,warmup_delay=0.,working_dir=None,shell=True,shell_args=None,uid=None,max_retry=5,gid=None,
	             send_hup=False,
	             stop_signal=signal.SIGTERM,stop_children=False,env=None,graceful_timeout=30.0,prereload_fn=None,rlimits=None,executable=None,
	             stdout_stream=None,stderr_stream=None,priority=0,loop=None,singleton=False,use_sockets=False,copy_env=False,copy_path=False,
	             max_age=0,max_age_variance=30,hooks=None,respawn=True,autostart=True,on_demand=False,virtualenv=None,stdin_socket=None,
	             close_child_stdin=True,close_child_stdout=False,close_child_stderr=False,virtualenv_py_ver=None,use_papa=False):
		# working_dir=working_dir or default_working_dir
		# virtualenv=virtualenv or virtualenv_root
		# warmup_delay=warmup_delay or watcher_warmup_delay
		# if watcher_respawn is False:
		# 	respawn=watcher_respawn
		if virtualenv: copy_env=True
		self.cmd=cmd
		# region Fields:
		self.name=name
		self.numprocesses=numprocesses
		self.warmup_delay=warmup_delay
		self.working_dir=working_dir
		self.shell=shell
		self.shell_args=shell_args
		self.uid=uid
		self.max_retry=max_retry
		self.gid=gid
		self.send_hup=send_hup
		self.stop_signal=stop_signal
		self.stop_children=stop_children
		self.env=env
		self.graceful_timeout=graceful_timeout
		self.prereload_fn=prereload_fn
		self.rlimits=rlimits
		self.executable=executable
		self.stdout_stream=stdout_stream
		self.stderr_stream=stderr_stream
		self.priority=priority
		self.loop=loop
		self.singleton=singleton
		self.use_sockets=use_sockets
		self.copy_env=copy_env
		self.copy_path=copy_path
		self.max_age=max_age
		self.max_age_variance=max_age_variance
		self.hooks=hooks
		self.respawn=respawn
		self.autostart=autostart
		self.on_demand=on_demand
		self.virtualenv=virtualenv
		self.stdin_socket=stdin_socket
		self.close_child_stdin=close_child_stdin
		self.close_child_stdout=close_child_stdout
		self.close_child_stderr=close_child_stderr
		self.virtualenv_py_ver=virtualenv_py_ver
		self.use_papa=use_papa
		# endregion
		
		TheWatcher.watchers.append(self.to_dict())
	
	def to_dict(self):
		return {k:v for k,v in self.__dict__.items() if v is not None}

class CeleryMan(object):
	
	def __init__(self,celery_path,app_name):
		self.celery_path=celery_path
		self.app_name=app_name
	
	def create_celery_worker(self,worker_name,Ofair=True,queues=None,queue_as_worker_name=True,concurrency=1,autoscale=None,log_level='info',
	                         **watcher_kwargs):
		"""
		This method creates celery worker.
		
		:param worker_name:
					Name of the worker. It will be provided as "-n <worker_name>" argument to celery worker.
		:param Ofair:
					see celery docs.
		:param queues:
					You can specify: 1) single queue (str), 2) list of strings as queue names.
		:param queue_as_worker_name:
					If "queues" is not provided, "worker_name" is used as queue name (by_default).
					If you don't want to specify any queue at all, then pass False.
		:param concurrency:
					Number of concurrent processses. see celery docs.
		:param autoscale:
					Tuple in the form (max,min) processes as specified in celery docs.
		:param log_level:
					Celery log level. By default = "info".
		:param watcher_kwargs:
					Other parameters are passed to TheWatcher.
		:return: Nothing.
		"""
		watcher_name=worker_name+'_watcher'
		parts=list()
		parts.append(f"{self.celery_path} worker -A {self.app_name} -l {log_level}")
		parts.append(f"-n {worker_name}")
		if Ofair: parts.append('-Ofair')
		if queues:
			if type(queues) is str:
				parts.append(f'-Q {queues}')
			else:
				parts.append(f'-Q {",".join(queues)}')
		elif queue_as_worker_name:
			parts.append(f'-Q {worker_name}')
		
		if autoscale:
			parts.append(f"--autoscale={','.join(map(str,autoscale))}")
		else:
			parts.append(f'-c {concurrency}')
		TheWatcher(watcher_name," ".join(parts),**watcher_kwargs)
		pass

def run_watchers():
	arbiter=circus.get_arbiter(TheWatcher.watchers)
	try:
		arbiter.start()
	finally:
		arbiter.stop()
	return

def main():
	"""
	TheWatcher is the main thing that you need for running your commands continuously.
	
		 TheWatcher(name='some_watcher_name',
		            cmd='Command that needs to be run continuously',
		            working_dir='specify_your_working_dir',
		            numprocesses=1,
		            respawn=True)
		 
		 TheWatcher(name='another_watcher_name',
		            cmd='Another command that needs to be run continuously',
		            working_dir='some_other_working_dir',
		            numprocesses=1,
		            respawn=False)
		 
		 run_watchers()
		 
		 That's it. When you call "run_watchers", all declared watchers will be launched in parallel.
		 
		 
	 
	If you need to run celery workers, you can use CeleryMan!:
	
		For example:
			
			cman=CeleryMan(celery_path=r'/usr/local/bin/celery',app_name='celery_worker')
			
			cman.create_celery_worker('get_image_date',autoscale=(18,3))
			cman.create_celery_worker('process_main_request',autoscale=(36,3))

			cman.create_celery_worker('download_products',concurrency=1)
			cman.create_celery_worker('atmospheric_correction_of_single_1c',concurrency=2)
	"""
	TheWatcher('testik','ls -lah',respawn=True) # plohoi primer. Eta commanda ne ranitsya continuously))).
	run_watchers()
	pass

if __name__=='__main__':
	main()



import setuptools
from lgblkb_tools.global_support import ConfigReader
from lgblkb_tools.global_support import Folder
from lgblkb_tools.log_support import create_logger,with_logging
import ruamel.yaml as raml

simple_logger=create_logger('setup_logs')

with open("README.md","r") as fh:
	long_description=fh.read()

def get_update_version(info_filepath):
	# yaml.dump(dict(version='0.0.8'),open(info_filepath,'w'))
	info_data=ConfigReader(info_filepath)
	current_version=[int(x) for x in info_data.version.obj.split('.')]
	current_version[-1]+=1
	info_data['version']='.'.join(map(str,current_version))
	return info_data

@simple_logger.with_logging()
def setup(version):
	setuptools.setup(
		name="lgblkb_tools",
		version=version,
		author="Dias Bakhtiyarov",
		author_email="dbakhtiyarov@nu.edu.kz",
		description="Some useful tools for everyday routine coding improvisation)",
		long_description=long_description,
		long_description_content_type="text/markdown",
		url="https://bitbucket.org/lgblkb/lgblkb_tools",
		packages=setuptools.find_packages(),
		classifiers=(
			"Programming Language :: Python :: 3.6",
			"License :: OSI Approved :: MIT License",
			"Operating System :: OS Independent",
			),
		install_requires=[
			'pandas',
			'python_log_indenter',
			'celery',
			'numpy',
			'geojson',
			'python_dateutil',
			'ruamel.yaml',
			'sklearn',
			'geojsonio',
			'matplotlib',
			'pyproj',
			'Shapely',
			'geojson',
			'scikit_learn',
			'more_itertools',
			'requests',
			'docker',
			'python-box',
			'sqlalchemy',
			'geoalchemy2',
			'psycopg2',
			'colorlog',
			'python-telegram-bot',
			'jsonpickle',
			'fabric',
			'circus',
			]
		)

@simple_logger.with_logging()
def main():
	# yaml=raml.YAML()
	# yaml.default_flow_style=False
	# yaml.dump(dict(version='1.1.2'),open(r'package_info.yaml','w'))
	# return
	base_dir=Folder(r'/home/lgblkb/PycharmProjects/lgblkb_tools/')
	build_dir=base_dir.create('build')
	dist_dir=base_dir.create('dist')
	info_filepath=r'/home/lgblkb/PycharmProjects/lgblkb_tools/package_info.yaml'
	info_data=get_update_version(info_filepath)
	simple_logger.info('New version: %s',info_data.version.obj)
	build_dir.delete()
	dist_dir.delete()
	setup(info_data.version.obj)
	info_data.update()
	
	pass

if __name__=='__main__':
	simple_logger.get_filepath(ext='log',include_depth=0).add_handler()
	main()



from abc import ABC,abstractmethod
import jsonpickle
import glob
from typing import Callable
import fnmatch
from datetime import datetime
import shutil
import os
import collections

get_name=lambda some_path:os.path.splitext(os.path.basename(some_path))[0]

def get_parent_dir(some_path,depth=1):
	for i in range(depth):
		some_path=os.path.dirname(os.path.abspath(some_path))
	return some_path

def create_path(depth: int,*paths):
	path=os.path.join(*paths)
	os.makedirs(get_parent_dir(path,depth),exist_ok=True)
	return path

def get_existing_path(paths,silent=False):
	for p in paths:
		if os.path.exists(p): return p
	if not silent:
		raise NotImplementedError('Could not find any existing path.')

def clear_folder(folder,remove_subdirs=True):
	for the_file in os.listdir(folder):
		file_path=os.path.join(folder,the_file)
		try:
			if os.path.isfile(file_path):
				os.unlink(file_path)
			elif remove_subdirs and os.path.isdir(file_path): shutil.rmtree(file_path)
		except Exception as e:
			print(e)

def get_full_paths_from(parent_dir: str):
	for file in os.listdir(parent_dir):
		yield os.path.join(parent_dir,file)

def get_splitted(path):
	folders=[]
	while 1:
		path,folder=os.path.split(path)
		
		if folder!="":
			folders.append(folder)
		else:
			if path!="":
				folders.append(path)
			break
	folders.reverse()
	return folders

class InfoDict(collections.OrderedDict):
	
	def get_dir_path(self,*paths):
		return os.path.join(*self.get_portions(),*paths)
	
	# def create_file_path(self,ext,dir_depth=1):
	# 	return create_info_path(self,ext=ext,*self.get_portions(),dir_depth=dir_depth)
	
	def get_portions(self):
		portions=list()
		for k,v in self.items():
			if v: portions.append(f"{k}={v}")
			else: portions.append(k)
		return portions
	
	def get_text(self,delim="_"):
		return delim.join(self.get_portions())
	
	def __str__(self):
		return self.get_text()

class JsonPicklable(ABC):
	
	def to_json(self):
		return jsonpickle.encode(self)

class Folder(JsonPicklable):
	
	def __init__(self,path='',parent=None):
		if not path: path=os.getcwd()
		elif not os.path.exists(path):
			assert os.path.splitext(os.path.split(path)[-1])[-1]=='',f"Please, use folder path as input. Provided path is: \n{path}"
			os.makedirs(path)
		elif not os.path.isdir(path):
			path=get_parent_dir(path,1)
		self.path=path
		self.parent: Folder=parent
		self.is_root=parent is None
		self.child_parts=list()
		self.name=get_name(path)
	
	def get_filepath(self,*name_portions,ext='',delim='_',include_depth=None,datetime_loc_index=None,**name_kwargs):
		parts=list()
		if include_depth not in [0,None]:
			if type(include_depth) is Callable:
				parent_parts=include_depth(get_splitted(self.path))
			elif type(include_depth) is int:
				parent_parts=get_splitted(self.path)[-include_depth:]
			elif type(include_depth) in [tuple,list]:
				parent_parts=[get_splitted(self.path)[-x] for x in include_depth]
			else:
				raise NotImplementedError(f'include_depth={include_depth}')
			parts.extend(parent_parts)
		# if include_parents:
		# 	parent=self
		# 	root_parts=list()
		# 	while True:
		# 		print('parent',parent)
		# 		print(parent.child_parts)
		# 		if parent.child_parts:
		# 			root_parts.append(parent.child_parts)
		# 		if parent.is_root: break
		# 		parent=parent.parent
		# 	if root_parts: parts.extend([y for x in root_parts[::-1] for y in x])
		part_portions=delim.join([str(x) for x in name_portions])
		if part_portions: parts.append(part_portions)
		part_kwargs=InfoDict(name_kwargs).get_portions()
		if part_kwargs: parts.extend(part_kwargs)
		if datetime_loc_index is not None: parts.insert(datetime_loc_index,datetime.now().strftime("%Y%m%d-%H:%M:%S"))
		assert parts,'Nothing if provided to create filepath.'
		return os.path.join(self.path,delim.join(parts).replace(' ',delim)+ext)
	
	def create(self,*child_folders,is_remote=False,**info_kwargs):
		parts=list()
		if child_folders: parts.extend(child_folders)
		if info_kwargs: parts.extend(InfoDict(info_kwargs).get_portions())
		assert parts,'Nothing is provided to create directory'
		self.child_parts=parts
		path=os.path.join(self.path,os.path.join(*self.child_parts))
		if not is_remote: path=create_path(0,path)
		return self.__class__(path,self)
	
	def delete(self):
		shutil.rmtree(self.path)
	
	def children(self,*paths):
		return glob.glob(os.path.join(self.path,*(paths or '*')))
	
	def __getitem__(self,item):
		return self.create(item)
	
	def __repr__(self):
		return f"Folder({self.path})"

class RemoteFolder(Folder):
	
	def __init__(self,path='',parent=None):
		super(RemoteFolder,self).__init__(path='',parent=parent)
		self.path=path
	
	def create(self,*child_folders,**info_kwargs):
		return super(RemoteFolder,self).create(*child_folders,is_remote=True,**info_kwargs)
	
	def delete(self):
		raise NotImplementedError
	
	def children(self,*paths):
		raise NotImplementedError

get_neighbor_path=lambda curr_path,filename:os.path.join(get_parent_dir(curr_path),filename)

def get_paths_matching(target_dir,match: str):
	items=os.listdir(target_dir)
	outs=list()
	for item in items:
		if match in item: outs.append(os.path.join(target_dir,item))
	return outs

def replace_ext(fpath,out_extension):
	base_path,ext=os.path.splitext(fpath)
	return base_path+out_extension

def find_file(directory,pattern):
	for root,dirs,files in os.walk(directory):
		for basename in files:
			if fnmatch.fnmatch(basename,pattern):
				filename=os.path.join(root,basename)
				return filename

def main():
	project_folder=Folder(__file__)
	logs_folder=project_folder.create('log_files')
	results_folder=project_folder.create('Results')
	qwe=InfoDict(name='didi',day='today',trial=1)
	# qwe=logs_folder.get_filepath(name='didi',day='today',trial=1,include_depth=2)
	print(qwe.get_dir_path())
	print(qwe)
	pass

if __name__=='__main__':
	main()
